# Consumidor de Datos de Kafka para la Plataforma EpI-PUMA

## Descripción

Este consumidor de Kafka fue creado como parte de la plataforma EpI-PUMA y tiene como objetivo reemplazar el proceso de carga de datos actuales de la plataforma. El proceso actual de carga de datos se realiza de la siguiente manera:
  - Bajar diariamente el archivo actualizado con los casos de COVID-19 desde el sitio de la Dirección General de Epidemiología.
  - Realizar un proceso de ETL y reconstruir la base de datos completa.

Este proceso presentó dos problemas importantes:
  - El proceso de ETL es lento y requiere de mucho tiempo de procesamiento (al rededor de 5 horas)
  - La DGE hace cambios en el conjunto de datos que incluye en sus archivos sin previo aviso, lo cual rompe el proceso de ETL y provoca que falten registros en la base de datos de la plataforma EpI-PUMA.

Por otro lado, la solución propuesta por este consumidor de Kafka realiza las siguientes tareas:
  - Tomar los datos de los casos de COVID-19 en México desde un tópico de Apache Kafka, el cual debe ser alimentado diariamente mediante el productor [KafkaCovid19](https://git.c3.unam.mx/alonso.ballesteros/KafkaCovid19).
  - Cargar los registros en la base de datos de PostgreSQL de la plataforma EpI-PUMA conforme se van generando en el tópico de Kafka.

El proceso propuesto tiene las siguientes ventajas respecto al proceso actual:
  - El proceso de carga de datos es mucho más rápido (al rededor de 5 minutos).
  - Los cambios en el contenido de los archivos de datos no afectan el proceso de carga de los registros, ya que son manejados independientemente por el productor [KafkaCovid19](https://git.c3.unam.mx/alonso.ballesteros/KafkaCovid19).
  - Al consumir de un tópico de Kafka, cualquier aplicación que requiera datos de este tipo, puede tener acceso a los datos de los casos de COVID-19 en México sin realizar un costoso proceso de ETL.

## Requisitos

Para poder ejecutar el proceso de carga de datos en Kafka es necesario tener instalados los siguientes paquetes en el sistema donde se ejecutará el proceso:
  - Python 3 y Pip

Además, en el mismo sistema o en uno dedicado se debe tener instalado y configurado un servidor de Apache Kafka con un tópico alimentado por el productor [KafkaCovid19](https://git.c3.unam.mx/alonso.ballesteros/KafkaCovid19).


## Instalación

Basta con ejecutar el siguiente comando para instalar las librerías necesarias para ejecutar el proceso de carga de datos en Kafka:

```sh
pip install -r requirements.txt
```

## Configuración

Los ajustes del conector se realizan en el archivo **conector.py**. Estos ajustes deben realizarse antes de ejecutar el conector, ya que de lo contrario fallará con una excepción.

### Base de datos de PostgreSQL

- Debe ser una base de datos de PostgreSQL con el esquema de la plataforma EpI-PUMA.

```python
# --------- Datos PostgreSQL ----------
PG_HOST       = 'base.datos.com'
PG_PUERTO     = 5433
PG_USUARIO    = 'usuario'
PG_CLAVE      = 'clave'
PG_BASE_DATOS = 'epi_puma'
```

### Apache Kafka

- En el campo de **KAFKA_BOOTSTRAP_SERVER** se debe omitir el protocolo **http://**.

- El campo **KAFKA_TOPIC** debe coincidir con el nombre del tópico que alimenta el productor [KafkaCovid19](https://git.c3.unam.mx/alonso.ballesteros/KafkaCovid19).

- Se sugiere que el campo **KAFKA_GROUP_ID** sea único para este consumidor de Kafka.

```python
# ------------ Datos Kafka ------------
KAFKA_BOOTSTRAP_SERVER = 'servidor.kafka:9092'
KAFKA_SCHEMA_REGISTRY  = 'http://servidor.kafka:8081'
KAFKA_GROUP_ID         = 'covid19_epi_puma'
KAFKA_TOPIC            = 'covid19'
KAFKA_PARTITION        = 0
```

### Conector

- El campo **TIEMPO_PURGA_REGISTROS** denota el tiempo que esperará el consumidor sin recibir nuevos registros antes de guardar los registros que tiene en memoria en la base de datos. El valor se expresa en segundos.
- El campo **LIMITE_CARGA_REGISTROS** denota el número máximo de registros que se guardarán en memoria antes de hacerlos persistentes en la base de datos, esto se realiza de manera transaccional. El valor se expresa en número de registros.

```python
TIEMPO_PURGA_REGISTROS = 60
LIMITE_CARGA_REGISTROS = 100000
```

## Conector

El conector posee una interfaz de línea de comandos que permite ejecutarlo de manera sencilla. La interfaz de línea de comandos se muestra a continuación:

```sh
python3 conector.py [-h] {estatus,iniciar,detener}

positional arguments:
  {estatus,iniciar,detener}

optional arguments:
  -h, --help            show this help message and exit
```

## Ejecución

Para ejecutar el consumidor de Kafka basta con ingresar el siguiente comando:

```sh
python3 conector.py iniciar
```

Con este comando el consumidor de Kafka se ejecutará en segundo plano y se mantendrá en ejecución hasta que se ingrese el siguiente comando:

```sh
python3 conector.py detener
```

Además, se puede obtener el estatus del consumidor de Kafka con el siguiente comando:

```sh
python3 conector.py estatus
```

El conector guardará un archivo de registros en el archivo **conector.log**. Este archivo se encuentra en el directorio donde se ejecuta el conector.